// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use std::sync::Arc;

use url::Url;

use crate::auth::application::port::input::ui::{
    errors::*,
    process_authorization::{ProcessAuthorizationInterface, ProcessAuthorizationInterfaceResp},
};
use crate::auth::application::port::out::db::{
    delete_oauth_access_token::DeleteOAuthAccessToken, delete_oauth_state::DeleteOAuthState,
    oauth_state_exists::OAuthStateExists, save_oauth_access_token::SaveOAuthAccessToken,
};
use crate::auth::application::port::out::forge::{
    get_username::GetUsername, request_access_token::RequestAccessToken,
};
use crate::auth::application::services::process_authorization_response::{
    command::ProcessAuthorizationResponseCommand, service::ProcessAuthorizationResponseService,
    ProcessAuthorizationResponseUseCase,
};

pub struct ProcessAuthorizationAdapter {
    oauth_state_exists_adapter: Arc<dyn OAuthStateExists>,
    delete_oauth_state_adapter: Arc<dyn DeleteOAuthState>,
    save_oauth_access_token_adapter: Arc<dyn SaveOAuthAccessToken>,
    delete_oauth_access_token_adapter: Arc<dyn DeleteOAuthAccessToken>,
    request_access_token_adapter: Arc<dyn RequestAccessToken>,
    get_username_adapter: Arc<dyn GetUsername>,
    process_authorization_response_redirect_uri: Url,
}

impl ProcessAuthorizationAdapter {
    pub fn new(
        oauth_state_exists_adapter: Arc<dyn OAuthStateExists>,
        delete_oauth_state_adapter: Arc<dyn DeleteOAuthState>,
        save_oauth_access_token_adapter: Arc<dyn SaveOAuthAccessToken>,
        delete_oauth_access_token_adapter: Arc<dyn DeleteOAuthAccessToken>,
        request_access_token_adapter: Arc<dyn RequestAccessToken>,
        get_username_adapter: Arc<dyn GetUsername>,
        process_authorization_response_redirect_uri: Url,
    ) -> Self {
        Self {
            oauth_state_exists_adapter,
            delete_oauth_state_adapter,
            save_oauth_access_token_adapter,
            delete_oauth_access_token_adapter,
            request_access_token_adapter,
            get_username_adapter,
            process_authorization_response_redirect_uri,
        }
    }
}

#[async_trait::async_trait]
impl ProcessAuthorizationInterface for ProcessAuthorizationAdapter {
    #[tracing::instrument(name = "web adapter process_authorization", skip(self, code))]
    async fn process_authorization(
        &self,
        code: String,
        state: String,
        oauth_provider: String,
        redirect_uri: Option<Url>,
    ) -> InUIResult<ProcessAuthorizationInterfaceResp> {
        let service = ProcessAuthorizationResponseService::new(
            self.oauth_state_exists_adapter.clone(),
            self.delete_oauth_state_adapter.clone(),
            self.save_oauth_access_token_adapter.clone(),
            self.delete_oauth_access_token_adapter.clone(),
            self.request_access_token_adapter.clone(),
            self.get_username_adapter.clone(),
            self.process_authorization_response_redirect_uri.clone(),
        );

        let cmd = ProcessAuthorizationResponseCommand::new_command(
            redirect_uri,
            state,
            code,
            oauth_provider,
        )?;
        let username = service.process_authorization_response(cmd).await?;
        Ok(ProcessAuthorizationInterfaceResp { username })
    }
}

#[cfg(test)]
mod tests {
    use super::*;
    use crate::auth::application::port::out::{
        db::{
            delete_oauth_access_token::tests::*, delete_oauth_state::tests::*,
            oauth_state_exists::tests::*, save_oauth_access_token::tests::*,
        },
        forge::{get_username::tests::*, request_access_token::tests::*},
    };
    use crate::tests::bdd::*;

    #[actix_web::test]
    async fn test_adapter() {
        let random_string = "foorand";
        let username = "processauthouathusername";
        let url = Url::parse("http://test_ui_req_auth_interface_adapter").unwrap();
        let oauth_provider = "test_ui_req_auth_interface_adapter";
        let code = "code";
        let state = "state";
        let mut redirect_uri = url.clone();
        redirect_uri.set_query(Some(&format!("state={random_string}")));

        // authorization response with redirect_uri = None
        {
            let adapter = ProcessAuthorizationAdapter::new(
                mock_oauth_state_exists(IS_CALLED_ONLY_ONCE, RETURNS_TRUE),
                mock_delete_oauth_state(IS_CALLED_ONLY_ONCE),
                mock_save_oauth_access_token(IS_CALLED_ONLY_ONCE),
                mock_delete_oauth_access_token(IS_CALLED_ONLY_ONCE),
                mock_request_access_token(IS_CALLED_ONLY_ONCE),
                mock_get_username(username.into(), IS_CALLED_ONLY_ONCE),
                url.clone(),
            );

            adapter
                .process_authorization(code.into(), state.into(), oauth_provider.into(), None)
                .await
                .unwrap();
        }

        // authorization response with redirect_uri = Some(Url)
        {
            let adapter = ProcessAuthorizationAdapter::new(
                mock_oauth_state_exists(IS_CALLED_ONLY_ONCE, RETURNS_TRUE),
                mock_delete_oauth_state(IS_CALLED_ONLY_ONCE),
                mock_save_oauth_access_token(IS_CALLED_ONLY_ONCE),
                mock_delete_oauth_access_token(IS_CALLED_ONLY_ONCE),
                mock_request_access_token(IS_CALLED_ONLY_ONCE),
                mock_get_username(username.into(), IS_CALLED_ONLY_ONCE),
                url.clone(),
            );

            adapter
                .process_authorization(
                    code.into(),
                    state.into(),
                    oauth_provider.into(),
                    Some(url.clone()),
                )
                .await
                .unwrap();
        }
    }
}
