// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use std::sync::Arc;

use super::types;
use crate::auth::adapter::out::forge::SupportedForges;
use crate::auth::application::port::input::forge::refresh_access_token::RefreshAccessTokenInputPort;
use crate::auth::application::port::input::ui::errors::*;
use crate::auth::application::services::refresh_access_token::command::*;
use actix_identity::Identity;
use actix_web::{get, http::header::ContentType, post, web, HttpResponse};

pub fn services(cfg: &mut web::ServiceConfig) {
    cfg.service(logged_in);
    cfg.service(refresh);
}

#[get("/protected/{forge}")]
#[tracing::instrument(name = "post log in in dummy page")]
async fn logged_in(forge: web::Path<SupportedForges>, _: Identity) -> InUIResult<HttpResponse> {
    let page = r#"
<!DOCTYPE html>
<html lang="en">
<head>
  <meta charset="UTF-8">
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
  <title>Document</title>
</head>
<body>
  <form action="{{submit_route}}" method="post">
    <button type="submit">
      Refresh Token
    </button>
  </form>
  
</body>
</html>
"#
    .replace("{{submit_route}}", &format!("/protected/{}/refresh", forge));
    Ok(HttpResponse::Ok()
        .insert_header(ContentType::html())
        .body(page))
}

#[post("/protected/{forge}/refresh")]
#[tracing::instrument(
    name = "refresh access token"
    skip(
        forges,
        adapter,
        id
    )
)]
async fn refresh(
    forges: types::WebForgeRepositoryInterface,
    adapter: web::Data<Arc<dyn RefreshAccessTokenInputPort>>,
    forge: web::Path<SupportedForges>,
    id: Identity,
) -> InUIResult<HttpResponse> {
    adapter
        .refresh_access_token(
            forges
                .get_forge_factory(&forge)
                .unwrap()
                .refresh_access_token_adapter(),
            RefreshAccessTokenCommand::new_command(id.id().unwrap(), forge.to_string()).unwrap(),
        )
        .await
        .unwrap();

    let page = r#"
<!DOCTYPE html>
<html lang="en">
<head>
  <meta charset="UTF-8">
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
  <title>Document</title>
</head>
<body>
Token Refreshed
  <form action="{{submit_route}}" method="post">
    <button type="submit">
      Refresh Token
    </button>
  </form>
  
</body>
</html>
"#
    .replace("{{submit_route}}", &format!("/protected/{}/refresh", forge));
    Ok(HttpResponse::Ok()
        .insert_header(ContentType::html())
        .body(page))
}
