// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use actix_identity::Identity;
use actix_web::{get, http::header, web, HttpMessage, HttpRequest, HttpResponse};
use serde::{Deserialize, Serialize};
use url::Url;

use super::types;
use crate::auth::adapter::out::forge::SupportedForges;
use crate::auth::application::port::input::ui::{
    errors::*, process_authorization::ProcessAuthorizationInterface,
};

pub fn services(cfg: &mut web::ServiceConfig) {
    cfg.service(handler);
}

#[derive(Debug, Clone, Eq, PartialEq, Ord, PartialOrd, Serialize, Deserialize)]
struct ProcessAuthorizationReqQueryParams {
    code: String,
    state: String,
    redirect_uri: Option<Url>,
}

#[get("/oauth/{forge}/authorize")]
#[tracing::instrument(
    name = "login page handler",
    skip(
        forges,
        oauth_state_exists_adapter,
        delete_oauth_state_adapter,
        save_oauth_access_token_adapter,
        delete_oauth_access_token_adapter,
        routes,
        settings,
        q,
    )
)]
async fn handler(
    forges: types::WebForgeRepositoryInterface,
    oauth_state_exists_adapter: types::WebOauthStateExists,
    delete_oauth_state_adapter: types::WebDeleteOauthState,
    save_oauth_access_token_adapter: types::WebSaveOAuthAccessToken,
    delete_oauth_access_token_adapter: types::WebDeleteOAuthAccessToken,
    routes: types::WebRouteRepository,
    settings: types::WebSettings,
    q: web::Query<ProcessAuthorizationReqQueryParams>,
    req: HttpRequest,
    forge: web::Path<SupportedForges>,
) -> InUIResult<HttpResponse> {
    let request_access_token_adapter = forges
        .get_forge_factory(&forge)
        .unwrap()
        .request_access_token_adapter();

    let get_username_adapter = forges
        .get_forge_factory(&forge)
        .unwrap()
        .get_username_adapter();

    let process_authorization_response_redirect_uri = Url::parse(&format!(
        "{}://{}{}",
        "http",
        &settings.server.domain,
        &routes.process_oauth_authorization_response(&forge)
    ))
    .map_err(|_| InUIError::InternalServerError)?;

    let web_adapter = super::adapter::ProcessAuthorizationAdapter::new(
        oauth_state_exists_adapter.as_ref().clone(),
        delete_oauth_state_adapter.as_ref().clone(),
        save_oauth_access_token_adapter.as_ref().clone(),
        delete_oauth_access_token_adapter.as_ref().clone(),
        request_access_token_adapter.clone(),
        get_username_adapter.clone(),
        process_authorization_response_redirect_uri,
    );
    let q = q.into_inner();

    let res = web_adapter
        .process_authorization(q.code, q.state, forge.to_string(), q.redirect_uri)
        .await?;
    Identity::login(&req.extensions(), res.username).unwrap();
    //Ok(HttpResponse::Ok().finish())
    Ok(HttpResponse::Found()
        .insert_header((header::LOCATION, format!("/protected/{}", &forge)))
        .finish())
}

#[cfg(test)]
mod tests {
    use std::sync::Arc;

    use super::*;
    use actix_identity::IdentityMiddleware;
    use actix_session::{storage::CookieSessionStore, SessionMiddleware};
    use actix_web::http::StatusCode;
    use actix_web::{cookie::Key, http::header::ContentType, test, App};

    use crate::auth::adapter::input::web::routes::RoutesRepository;
    use crate::auth::adapter::out::forge::forge_factory::{
        ForgeAdapterFactoryInterface, MockForgeAdapterFactoryInterface,
    };
    use crate::auth::adapter::out::forge::forge_repository::MockForgeRepositoryInterface;
    use crate::auth::application::port::out::{
        db::{
            delete_oauth_access_token::tests::*, delete_oauth_state::tests::*,
            oauth_state_exists::tests::*, save_oauth_access_token::tests::*,
        },
        forge::{get_username::tests::*, request_access_token::tests::*},
    };
    use crate::tests::bdd::*;

    #[actix_web::test]
    async fn test_ui_handler_process_oauth_authorization_forgejo() {
        let username = "foorand";
        let state = "hstate";
        let code = "hcode";
        let settings = crate::settings::Settings::new().unwrap();

        let mock_delete_oauth_state =
            types::WebDeleteOauthState::new(mock_delete_oauth_state(IS_CALLED_ONLY_ONCE));

        let mock_oauth_state_exists = types::WebOauthStateExists::new(mock_oauth_state_exists(
            IS_CALLED_ONLY_ONCE,
            RETURNS_TRUE,
        ));

        let mock_save_oauth_access_token =
            types::WebSaveOAuthAccessToken::new(mock_save_oauth_access_token(IS_CALLED_ONLY_ONCE));

        let mock_delete_oauth_access_token = types::WebDeleteOAuthAccessToken::new(
            mock_delete_oauth_access_token(IS_CALLED_ONLY_ONCE),
        );

        let mock_forges = {
            let mut mock_forge_factory = MockForgeAdapterFactoryInterface::default();
            mock_forge_factory
                .expect_request_access_token_adapter()
                .times(IS_CALLED_ONLY_ONCE.unwrap())
                .returning(|| mock_request_access_token(IS_CALLED_ONLY_ONCE));

            mock_forge_factory
                .expect_get_username_adapter()
                .times(IS_CALLED_ONLY_ONCE.unwrap())
                .returning(|| mock_get_username(username.into(), IS_CALLED_ONLY_ONCE));

            let mock_forge_factory: Arc<dyn ForgeAdapterFactoryInterface> =
                Arc::new(mock_forge_factory);

            let mut mock_forges = MockForgeRepositoryInterface::default();
            mock_forges
                .expect_get_forge_factory()
                .times(IS_CALLED_ONLY_TWICE.unwrap())
                .returning(move |_| Some(mock_forge_factory.clone()));

            types::WebForgeRepositoryInterface::new(Arc::new(mock_forges))
        };

        let routes = RoutesRepository::default();

        let process_authorization_response_redirect_uri = Url::parse(&format!(
            "{}://{}{}",
            "http",
            &settings.server.domain,
            &routes.process_oauth_authorization_response(&SupportedForges::Forgejo)
        ))
        .unwrap();

        let secret_key = Key::from(settings.server.cookie_secret.as_bytes());
        let app = test::init_service(
            App::new()
                .wrap(IdentityMiddleware::default())
                .wrap(SessionMiddleware::new(
                    CookieSessionStore::default(),
                    secret_key.clone(),
                ))
                .wrap(tracing_actix_web::TracingLogger::default())
                .app_data(mock_oauth_state_exists)
                .app_data(mock_save_oauth_access_token)
                .app_data(mock_delete_oauth_access_token)
                .app_data(mock_forges)
                .app_data(mock_delete_oauth_state)
                .app_data(web::Data::new(Arc::new(routes.clone())))
                .app_data(web::Data::new(settings.clone()))
                .service(handler),
        )
        .await;

        let path = {
            let mut u = process_authorization_response_redirect_uri.clone();
            u.set_path("/oauth/forgejo/authorize");
            u.set_query(Some(&format!(
                "state={state}&code={code}&redirect_uri={}",
                process_authorization_response_redirect_uri
            )));
            format!("{}?{}", u.path(), u.query().unwrap())
        };
        println!("path: {path}");
        let req = test::TestRequest::get()
            .uri(&path)
            .insert_header(ContentType::html())
            .to_request();
        let resp = test::call_service(&app, req).await;
        let status = resp.status();
        assert_eq!(status, StatusCode::FOUND);
    }
}
