// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use std::sync::Arc;

use actix_web::web;
use sqlx::postgres::PgPool;

pub mod input;
pub mod out;

use crate::auth::application::port::input::forge::refresh_access_token::RefreshAccessTokenInputPort;
use crate::settings;
use out::db::postgres::DBOutPostgresAdapter;
use out::forge::{forge_repository::ForgeRepository, forgejo::Forgejo, github::Github};

use input::{forge::refresh_access_token::RefreshAccessTokenInputAdapter, web::types::*};

pub fn load_adapters(
    pool: PgPool,
    settings: &settings::Settings,
) -> impl FnOnce(&mut web::ServiceConfig) {
    let db = DBOutPostgresAdapter::new(pool);
    let save_oauth_state_adapter: WebSaveOauthState = WebSaveOauthState::new(Arc::new(db.clone()));
    let delete_oauth_state_adapter: WebDeleteOauthState =
        WebDeleteOauthState::new(Arc::new(db.clone()));
    let save_oauth_access_token: WebSaveOAuthAccessToken =
        WebSaveOAuthAccessToken::new(Arc::new(db.clone()));
    let get_oauth_access_token: WebGetOAuthAccessToken =
        WebGetOAuthAccessToken::new(Arc::new(db.clone()));
    let delete_oauth_access_token: WebDeleteOAuthAccessToken =
        WebDeleteOAuthAccessToken::new(Arc::new(db.clone()));

    let oauth_state_exists_adapter: WebOauthStateExists =
        WebOauthStateExists::new(Arc::new(db.clone()));
    let refresh_access_token_input_adapter: Arc<dyn RefreshAccessTokenInputPort> =
        Arc::new(RefreshAccessTokenInputAdapter::new(
            Arc::new(db.clone()),
            Arc::new(db.clone()),
            Arc::new(db.clone()),
        ));

    let forgejo = Forgejo::new(
        settings.forges.forgejo.url.clone(),
        settings.forges.forgejo.client_id.clone(),
        settings.forges.forgejo.client_secret.clone(),
        refresh_access_token_input_adapter.clone(),
    );

    let github = Github::new(
        settings.forges.github.url.clone(),
        settings.forges.github.api_url.clone(),
        settings.forges.github.client_id.clone(),
        settings.forges.github.client_secret.clone(),
        refresh_access_token_input_adapter.clone(),
    );

    let forge_repository_interface =
        WebForgeRepositoryInterface::new(Arc::new(ForgeRepository::new(forgejo, github)));

    let s = WebSettings::new(settings.clone());

    let f = move |cfg: &mut web::ServiceConfig| {
        cfg.app_data(save_oauth_state_adapter);
        cfg.app_data(delete_oauth_state_adapter);
        cfg.app_data(save_oauth_access_token);
        cfg.app_data(get_oauth_access_token);
        cfg.app_data(delete_oauth_access_token);
        cfg.app_data(oauth_state_exists_adapter);

        cfg.app_data(forge_repository_interface);

        cfg.app_data(web::Data::new(refresh_access_token_input_adapter));

        cfg.app_data(s);

        cfg.configure(input::web::load_ctx());
    };

    Box::new(f)
}
