// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use super::DBOutPostgresAdapter;
use crate::auth::application::port::out::db::{
    delete_oauth_access_token::DeleteOAuthAccessToken, errors::OutDBPortResult,
};

#[async_trait::async_trait]
impl DeleteOAuthAccessToken for DBOutPostgresAdapter {
    async fn delete_oauth_access_token(
        &self,
        username: &str,
        oauth_provider: &str,
    ) -> OutDBPortResult<()> {
        sqlx::query!(
            "
            DELETE FROM
                oauth_access_token
            WHERE
                username = $1
            AND
                oauth_provider = $2;",
            username,
            oauth_provider
        )
        .execute(&self.pool)
        .await?;
        Ok(())
    }
}

#[cfg(test)]
mod tests {
    use super::*;
    use crate::auth::application::port::out::db::{
        get_oauth_access_token::GetOAuthAccessToken, save_oauth_access_token::SaveOAuthAccessToken,
    };
    use crate::auth::domain::OAuthAccessToken;

    #[actix_rt::test]
    async fn test_postgres_delete_oauth_access_token() {
        let oauth_provider = "oauthprovitestpostgres";
        let username = "test_postgres_save_oauth_access_token";
        let old_access_token = OAuthAccessToken {
            refresh_token: "asdfasdfarefresh".into(),
            ..Default::default()
        };

        let settings = crate::settings::tests::get_settings().await;

        let db = super::DBOutPostgresAdapter::new(
            sqlx::postgres::PgPool::connect(&settings.database.url)
                .await
                .unwrap(),
        );

        db.save_oauth_access_token(username, oauth_provider, &old_access_token)
            .await
            .unwrap();

        db.delete_oauth_access_token(username, oauth_provider)
            .await
            .unwrap();

        assert!(db
            .get_oauth_access_token(username, oauth_provider)
            .await
            .is_err());

        settings.drop_db().await;
    }
}
