// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use serde::{Deserialize, Serialize};
use sqlx::types::time::OffsetDateTime as SqlxOffsetDateTime;
use time::OffsetDateTime;

use super::DBOutPostgresAdapter;
use crate::auth::application::port::out::db::{
    errors::OutDBPortResult, get_oauth_access_token::GetOAuthAccessToken,
};
use crate::auth::domain::OAuthAccessToken;

#[derive(Debug, Clone, Serialize, Deserialize, PartialEq, Eq, PartialOrd, Ord)]
struct InternalOAuthAccessToken {
    refresh_token: String,
    access_token: String,
    token_type: String,
    expires_in: i64,
    created_at: SqlxOffsetDateTime,
}

impl From<InternalOAuthAccessToken> for OAuthAccessToken {
    fn from(v: InternalOAuthAccessToken) -> Self {
        Self::new(
            v.refresh_token,
            v.access_token,
            v.token_type,
            v.expires_in as u64,
            OffsetDateTime::from_unix_timestamp(v.created_at.unix_timestamp()).unwrap(),
        )
    }
}

#[async_trait::async_trait]
impl GetOAuthAccessToken for DBOutPostgresAdapter {
    async fn get_oauth_access_token(
        &self,
        username: &str,
        oauth_provider: &str,
    ) -> OutDBPortResult<OAuthAccessToken> {
        let res = sqlx::query_as!(
            InternalOAuthAccessToken,
            "SELECT 
                access_token, token_type, expires_in, refresh_token, created_at
            FROM
                oauth_access_token
            WHERE
                username = $1
            AND
                oauth_provider = $2;",
            username,
            oauth_provider
        )
        .fetch_one(&self.pool)
        .await?;
        Ok(res.into())
    }
}

#[cfg(test)]
mod tests {
    use super::*;
    use crate::auth::application::port::out::db::save_oauth_access_token::SaveOAuthAccessToken;

    #[actix_rt::test]
    async fn test_postgres_get_oauth_access_token() {
        let oauth_provider = "oauthprovitestpostgres";
        let username = "test_postgres_save_oauth_access_token";
        let access_token = OAuthAccessToken {
            refresh_token: "asdfasdfarefresh".into(),
            ..Default::default()
        };

        let settings = crate::settings::tests::get_settings().await;

        let db = super::DBOutPostgresAdapter::new(
            sqlx::postgres::PgPool::connect(&settings.database.url)
                .await
                .unwrap(),
        );

        db.save_oauth_access_token(username, oauth_provider, &access_token)
            .await
            .unwrap();

        assert_eq!(
            db.get_oauth_access_token(username, oauth_provider)
                .await
                .unwrap(),
            access_token
        );

        settings.drop_db().await;
    }
}
