// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use super::DBOutPostgresAdapter;
use crate::auth::application::port::out::db::{
    errors::OutDBPortResult, save_oauth_access_token::SaveOAuthAccessToken,
};
use crate::auth::domain::OAuthAccessToken;

#[async_trait::async_trait]
impl SaveOAuthAccessToken for DBOutPostgresAdapter {
    async fn save_oauth_access_token(
        &self,
        username: &str,
        oauth_provider: &str,
        access_token: &OAuthAccessToken,
    ) -> OutDBPortResult<()> {
        sqlx::query!(
            "INSERT INTO oauth_access_token (
                    access_token, token_type, expires_in,
                    refresh_token, oauth_provider, username, created_at
                ) VALUES ($1, $2, $3, $4, $5, $6, $7)",
            &access_token.access_token,
            &access_token.token_type,
            access_token.expires_in as i64,
            &access_token.refresh_token,
            oauth_provider,
            username,
            &access_token.created_at
        )
        .execute(&self.pool)
        .await?;
        Ok(())
    }
}

#[cfg(test)]
mod tests {
    use super::*;
    use crate::auth::application::port::out::db::errors::*;

    #[actix_rt::test]
    async fn test_postgres_save_oauth_access_token() {
        let oauth_provider = "oauthprovitestpostgres";
        let username = "test_postgres_save_oauth_access_token";
        let access_token = OAuthAccessToken {
            refresh_token: "asdfasdfarefresh".into(),
            ..Default::default()
        };

        let settings = crate::settings::tests::get_settings().await;

        let db = super::DBOutPostgresAdapter::new(
            sqlx::postgres::PgPool::connect(&settings.database.url)
                .await
                .unwrap(),
        );

        db.save_oauth_access_token(username, oauth_provider, &access_token)
            .await
            .unwrap();
        assert_eq!(
            db.save_oauth_access_token(username, oauth_provider, &access_token)
                .await
                .err(),
            Some(OutDBPortError::DuplicateAccessToken)
        );

        settings.drop_db().await;
    }
}
