// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use url::Url;

use super::DBOutPostgresAdapter;
use crate::auth::application::port::out::db::{
    errors::OutDBPortResult, save_oauth_state::SaveOAuthState,
};

#[async_trait::async_trait]
impl SaveOAuthState for DBOutPostgresAdapter {
    async fn save_oauth_state(
        &self,
        state: &str,
        oauth_provider: &str,
        redirect_uri: &Url,
    ) -> OutDBPortResult<()> {
        sqlx::query!(
            "INSERT INTO oauth_state (state, oauth_provider, redirect_uri) VALUES ($1, $2, $3)",
            state,
            oauth_provider,
            redirect_uri.as_str()
        )
        .execute(&self.pool)
        .await?;
        Ok(())
    }
}

#[cfg(test)]
mod tests {
    use super::*;
    use crate::auth::application::port::out::db::errors::*;

    #[actix_rt::test]
    async fn test_postgres_save_oauth_state() {
        let state = "statetestpostgres";
        let oauth_provider = "oauthprovitestpostgres";
        let redirect_uri = Url::parse("https://oauthprovitestpostgres").unwrap();

        let settings = crate::settings::tests::get_settings().await;

        let db = super::DBOutPostgresAdapter::new(
            sqlx::postgres::PgPool::connect(&settings.database.url)
                .await
                .unwrap(),
        );

        db.save_oauth_state(state, oauth_provider, &redirect_uri)
            .await
            .unwrap();

        assert_eq!(
            db.save_oauth_state(state, oauth_provider, &redirect_uri)
                .await
                .err(),
            Some(OutDBPortError::DuplicateState)
        );

        settings.drop_db().await;
    }
}
