// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use std::sync::Arc;

use mockall::predicate::*;
use mockall::*;

use super::{forgejo::Forgejo, github::Github};
use crate::auth::application::port::out::forge::{
    get_username::GetUsername, oauth_auth_req_uri::OAuthAuthReqUri,
    refresh_access_token::RefreshAccessToken, request_access_token::RequestAccessToken,
};

#[automock]
pub trait ForgeAdapterFactoryInterface: Send + Sync {
    fn oauth_auth_req_uri_adapter(&self) -> Arc<dyn OAuthAuthReqUri>;
    fn request_access_token_adapter(&self) -> Arc<dyn RequestAccessToken>;
    fn get_username_adapter(&self) -> Arc<dyn GetUsername>;
    fn refresh_access_token_adapter(&self) -> Arc<dyn RefreshAccessToken>;
}

#[derive(Clone)]
pub struct ForgejoAdapterFactory {
    forgejo: Arc<Forgejo>,
}

impl ForgeAdapterFactoryInterface for ForgejoAdapterFactory {
    fn oauth_auth_req_uri_adapter(&self) -> Arc<dyn OAuthAuthReqUri> {
        self.forgejo.clone()
    }
    fn request_access_token_adapter(&self) -> Arc<dyn RequestAccessToken> {
        self.forgejo.clone()
    }
    fn get_username_adapter(&self) -> Arc<dyn GetUsername> {
        self.forgejo.clone()
    }
    fn refresh_access_token_adapter(&self) -> Arc<dyn RefreshAccessToken> {
        self.forgejo.clone()
    }
}

impl ForgejoAdapterFactory {
    pub fn new(forgejo: Forgejo) -> Self {
        Self {
            forgejo: Arc::new(forgejo),
        }
    }
}

#[derive(Clone)]
pub struct GithubAdapterFactory {
    github: Arc<Github>,
}

impl ForgeAdapterFactoryInterface for GithubAdapterFactory {
    fn oauth_auth_req_uri_adapter(&self) -> Arc<dyn OAuthAuthReqUri> {
        self.github.clone()
    }
    fn request_access_token_adapter(&self) -> Arc<dyn RequestAccessToken> {
        self.github.clone()
    }
    fn get_username_adapter(&self) -> Arc<dyn GetUsername> {
        self.github.clone()
    }
    fn refresh_access_token_adapter(&self) -> Arc<dyn RefreshAccessToken> {
        self.github.clone()
    }
}

impl GithubAdapterFactory {
    pub fn new(github: Github) -> Self {
        Self {
            github: Arc::new(github),
        }
    }
}
