// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use std::collections::HashMap;
use std::sync::Arc;

use mockall::predicate::*;
use mockall::*;

use super::{
    forge_factory::ForgeAdapterFactoryInterface,
    forge_factory::{ForgejoAdapterFactory, GithubAdapterFactory},
    forgejo::Forgejo,
    github::Github,
    SupportedForges,
};

#[automock]
pub trait ForgeRepositoryInterface: Send + Sync {
    fn add_forge(
        &mut self,
        name: SupportedForges,
        forge_factory: Arc<dyn ForgeAdapterFactoryInterface>,
    );

    fn get_supported_forge_str(&self) -> Vec<String>;

    fn get_supported_forges(&self) -> Vec<SupportedForges>;

    fn get_forge_factory(
        &self,
        name: &SupportedForges,
    ) -> Option<Arc<dyn ForgeAdapterFactoryInterface>>;
}

#[derive(Clone, Default)]
pub struct ForgeRepository {
    forges: HashMap<SupportedForges, Arc<dyn ForgeAdapterFactoryInterface>>,
}

impl ForgeRepositoryInterface for ForgeRepository {
    fn add_forge(
        &mut self,
        name: SupportedForges,
        forge_factory: Arc<dyn ForgeAdapterFactoryInterface>,
    ) {
        self.forges.insert(name, forge_factory);
    }

    fn get_supported_forge_str(&self) -> Vec<String> {
        self.forges
            .clone()
            .into_keys()
            .map(|v| v.to_string())
            .collect()
    }

    fn get_supported_forges(&self) -> Vec<SupportedForges> {
        self.forges.clone().into_keys().collect()
    }

    fn get_forge_factory(
        &self,
        name: &SupportedForges,
    ) -> Option<Arc<dyn ForgeAdapterFactoryInterface>> {
        self.forges.get(name).cloned()
    }
}

impl ForgeRepository {
    pub fn new(forgejo: Forgejo, github: Github) -> Self {
        let forgejo_adapter = ForgejoAdapterFactory::new(forgejo);
        let github_adapter = GithubAdapterFactory::new(github);
        let mut s = Self::default();
        s.add_forge(SupportedForges::Forgejo, Arc::new(forgejo_adapter));
        s.add_forge(SupportedForges::Github, Arc::new(github_adapter));
        s
    }
}
