// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use serde::{Deserialize, Serialize};

use super::Forgejo;
use crate::auth::application::port::out::forge::{errors::*, get_username::GetUsername};
use crate::auth::domain::OAuthAccessToken;

#[derive(Clone, Debug, Eq, PartialEq, PartialOrd, Ord, Serialize, Deserialize)]
struct ForgejoUserResponse {
    login: String,
}

#[async_trait::async_trait]
impl GetUsername for Forgejo {
    async fn get_username(&self, access_token: &OAuthAccessToken) -> OutForgePortResult<String> {
        let u = {
            let mut u = self.url().to_owned();
            u.set_path("/api/v1/user");
            u
        };
        let res = self
            .http_client
            .get(u)
            .header(
                reqwest::header::AUTHORIZATION,
                format!("token {}", access_token.access_token),
            )
            .send()
            .await
            .unwrap();

        let res: ForgejoUserResponse = res.json().await.unwrap();
        Ok(res.login)
    }
}

#[cfg(test)]
mod tests {
    use super::*;
    use crate::auth::application::port::input::forge::refresh_access_token::tests::*;

    use url::Url;

    #[actix_rt::test]
    async fn test_forgejo_get_username() {
        let client_id = "statetestpostgres";
        let client_secret = "oauthprovitestpostgres";
        let access_token = "access_token_uuu";
        let username = "access_token_user";

        let mut settings = crate::settings::tests::get_settings().await;

        let mut srv = mockito::Server::new_async().await;
        settings.forges.forgejo.url = Url::parse(&srv.url()).unwrap();

        let f = Forgejo::new(
            settings.forges.forgejo.url.clone(),
            client_id.into(),
            client_secret.into(),
            mock_refresh_access_token_usecase_factory(None),
        );

        let mock = {
            let resp = ForgejoUserResponse {
                login: username.into(),
            };
            srv.mock("GET", "/api/v1/user")
                .with_status(200)
                .match_header("Authorization", format!("token {access_token}").as_str())
                .with_header("Content-Type", "application/json")
                .with_body(serde_json::to_string(&resp).unwrap())
                .create_async()
                .await
        };

        let req = OAuthAccessToken {
            access_token: access_token.into(),
            ..Default::default()
        };

        assert_eq!(f.get_username(&req).await.unwrap(), username);
        mock.assert_async().await;
    }
}
