// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use serde::{Deserialize, Serialize};

use super::request_access_token::ForgejoAccessTokenResponse;
use super::Forgejo;
use crate::auth::application::port::out::forge::{
    errors::*, refresh_access_token::RefreshAccessToken,
};
use crate::auth::domain::OAuthAccessToken;

#[derive(Clone, Debug, Eq, PartialEq, PartialOrd, Ord, Serialize, Deserialize)]
struct ForgejoAccessTokenRefreshRequest {
    refresh_token: String,
    client_id: String,
    client_secret: String,
    grant_type: String,
}

impl ForgejoAccessTokenRefreshRequest {
    pub fn new(refresh_token: String, forgejo: &Forgejo) -> Self {
        Self {
            client_id: forgejo.client_id.clone(),
            client_secret: forgejo.client_secret.clone(),
            grant_type: "refresh_token".into(),
            refresh_token,
        }
    }
}

#[async_trait::async_trait]
impl RefreshAccessToken for Forgejo {
    async fn refresh_access_token(
        &self,
        old_access_token: &OAuthAccessToken,
    ) -> OutForgePortResult<OAuthAccessToken> {
        let u = {
            let mut u = self.url().to_owned();
            u.set_path("/login/oauth/access_token");
            u
        };

        let payload =
            ForgejoAccessTokenRefreshRequest::new(old_access_token.refresh_token.clone(), self);
        let res: ForgejoAccessTokenResponse = self
            .http_client
            .post(u)
            .json(&payload)
            .send()
            .await
            .unwrap()
            .json()
            .await
            .unwrap();
        Ok(res.into())
    }
}

#[cfg(test)]
mod tests {
    use url::Url;

    use super::*;

    use crate::auth::application::port::input::forge::refresh_access_token::tests::*;

    #[actix_rt::test]
    async fn test_forgejo_refresh_access_token() {
        let client_id = "statetestpostgres";
        let client_secret = "oauthprovitestpostgres";
        let refresh_token = "refresh_token";

        let mut settings = crate::settings::tests::get_settings().await;

        let mut srv = mockito::Server::new_async().await;
        settings.forges.forgejo.url = Url::parse(&srv.url()).unwrap();

        let old_access_token = OAuthAccessToken {
            access_token: "foo".into(),
            refresh_token: refresh_token.into(),
            expires_in: 9999,
            ..Default::default()
        };

        let new_resp_data = ForgejoAccessTokenResponse {
            access_token: "new_access_token".into(),
            refresh_token: "new_refresh_token".into(),
            expires_in: 9999,
            ..Default::default()
        };

        let f = Forgejo::new(
            settings.forges.forgejo.url.clone(),
            client_id.into(),
            client_secret.into(),
            mock_refresh_access_token_usecase_factory(None),
        );

        let mock = {
            let payload =
                ForgejoAccessTokenRefreshRequest::new(old_access_token.refresh_token.clone(), &f);

            srv.mock("POST", "/login/oauth/access_token")
                .with_status(200)
                .match_header("content-type", "application/json")
                .match_body(mockito::Matcher::PartialJsonString(
                    serde_json::to_string(&payload).unwrap(),
                ))
                .with_header("Content-Type", "application/json")
                .with_body(serde_json::to_string(&new_resp_data).unwrap())
                .create_async()
                .await
        };

        let new_access_token = f.refresh_access_token(&old_access_token).await.unwrap();
        let expected_access_token: OAuthAccessToken = new_resp_data.into();
        assert_ne!(old_access_token, new_access_token);
        assert_eq!(expected_access_token, new_access_token);
        mock.assert_async().await;
    }
}
