// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use serde::{Deserialize, Serialize};
use time::OffsetDateTime;
use url::Url;

use super::Forgejo;
use crate::auth::application::port::out::forge::{
    errors::OutForgePortResult, request_access_token::RequestAccessToken,
};
use crate::auth::domain::OAuthAccessToken;

#[derive(Clone, Debug, Default, Eq, PartialEq, PartialOrd, Ord, Serialize, Deserialize)]
pub struct ForgejoAccessTokenResponse {
    pub refresh_token: String,
    pub access_token: String,
    pub token_type: String,
    pub expires_in: u64,
}

impl From<ForgejoAccessTokenResponse> for OAuthAccessToken {
    fn from(v: ForgejoAccessTokenResponse) -> Self {
        let created_at = OffsetDateTime::now_utc();
        Self::new(
            v.refresh_token,
            v.access_token,
            v.token_type,
            v.expires_in,
            created_at,
        )
    }
}

#[derive(Clone, Debug, Eq, PartialEq, PartialOrd, Ord, Serialize, Deserialize)]
struct ForgejoAccessTokenRequest {
    code: String,
    client_id: String,
    client_secret: String,
    grant_type: String,
    redirect_uri: Url,
}

impl ForgejoAccessTokenRequest {
    pub fn new(code: String, redirect_uri: Url, forgejo: &Forgejo) -> Self {
        Self {
            code,
            client_id: forgejo.client_id.clone(),
            client_secret: forgejo.client_secret.clone(),
            grant_type: "authorization_code".into(),
            redirect_uri,
        }
    }
}

#[async_trait::async_trait]
impl RequestAccessToken for Forgejo {
    async fn request_access_token(
        &self,
        code: String,
        redirect_uri: Url,
    ) -> OutForgePortResult<OAuthAccessToken> {
        let u = {
            let mut u = self.url().to_owned();
            u.set_path("/login/oauth/access_token");
            u
        };

        let payload = ForgejoAccessTokenRequest::new(code, redirect_uri, self);
        let res: ForgejoAccessTokenResponse = self
            .http_client
            .post(u)
            .json(&payload)
            .send()
            .await
            .unwrap()
            .json()
            .await
            .unwrap();
        Ok(res.into())
    }
}

#[cfg(test)]
mod tests {
    use super::*;

    use crate::auth::application::port::input::forge::refresh_access_token::tests::*;

    #[actix_rt::test]
    async fn test_forgejo_request_access_token() {
        let client_id = "statetestpostgres";
        let client_secret = "oauthprovitestpostgres";
        let code = "oauthprovitestpostgres";
        let redirect_uri = Url::parse("https://oauthprovitestpostgres").unwrap();

        let mut settings = crate::settings::tests::get_settings().await;

        let mut srv = mockito::Server::new_async().await;
        settings.forges.forgejo.url = Url::parse(&srv.url()).unwrap();

        let data = ForgejoAccessTokenResponse {
            access_token: "foo".into(),
            expires_in: 9999,
            ..Default::default()
        };

        let f = Forgejo::new(
            settings.forges.forgejo.url.clone(),
            client_id.into(),
            client_secret.into(),
            mock_refresh_access_token_usecase_factory(None),
        );

        let mock = {
            let payload = ForgejoAccessTokenRequest::new(code.into(), redirect_uri.clone(), &f);

            srv.mock("POST", "/login/oauth/access_token")
                .with_status(200)
                .match_header("content-type", "application/json")
                .match_body(mockito::Matcher::PartialJsonString(
                    serde_json::to_string(&payload).unwrap(),
                ))
                .with_header("Content-Type", "application/json")
                .with_body(serde_json::to_string(&data).unwrap())
                .create_async()
                .await
        };

        let access_token = f
            .request_access_token(code.into(), redirect_uri)
            .await
            .unwrap();
        let expected_access_token: OAuthAccessToken = data.into();
        assert_eq!(expected_access_token, access_token);
        mock.assert_async().await;
    }
}
