// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use serde::{Deserialize, Serialize};

use super::Github;
use crate::auth::application::port::out::forge::{errors::*, get_username::GetUsername};
use crate::auth::domain::OAuthAccessToken;

#[derive(Clone, Debug, Eq, PartialEq, PartialOrd, Ord, Serialize, Deserialize)]
struct GithubUserResponse {
    login: String,
}

#[async_trait::async_trait]
impl GetUsername for Github {
    async fn get_username(&self, access_token: &OAuthAccessToken) -> OutForgePortResult<String> {
        let u = {
            let mut u = self.api_url.clone();
            u.set_path("/user");
            u
        };
        let req = self
            .http_client
            .get(u)
            .bearer_auth(&access_token.access_token)
            .header(reqwest::header::USER_AGENT, "forgeflux.org")
            .header(reqwest::header::ACCEPT, "application/json");
        let res = req.send().await.unwrap();

        let res: GithubUserResponse = res.json().await.unwrap();
        Ok(res.login)
    }
}

#[cfg(test)]
mod tests {
    use super::*;
    use crate::auth::application::port::input::forge::refresh_access_token::tests::*;

    use url::Url;

    #[actix_rt::test]
    async fn test_github_get_username() {
        let client_id = "statetestpostgres";
        let client_secret = "oauthprovitestpostgres";
        let access_token = "access_token_uuu";
        let username = "access_token_user";

        let mut settings = crate::settings::tests::get_settings().await;

        let mut srv = mockito::Server::new_async().await;
        settings.forges.github.api_url = Url::parse(&srv.url()).unwrap();

        let f = Github::new(
            settings.forges.github.url.clone(),
            settings.forges.github.api_url.clone(),
            client_id.into(),
            client_secret.into(),
            mock_refresh_access_token_usecase_factory(None),
        );

        let mock = {
            let resp = GithubUserResponse {
                login: username.into(),
            };
            srv.mock("GET", "/user")
                .with_status(200)
                .match_header("Authorization", format!("Bearer {access_token}").as_str())
                .match_header(reqwest::header::USER_AGENT.to_string(), "forgeflux.org")
                .with_header("Content-Type", "application/json")
                .with_body(serde_json::to_string(&resp).unwrap())
                .create_async()
                .await
        };

        let req = OAuthAccessToken {
            access_token: access_token.into(),
            ..Default::default()
        };

        assert_eq!(f.get_username(&req).await.unwrap(), username);
        mock.assert_async().await;
    }
}
