// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use serde::{Deserialize, Serialize};

use super::request_access_token::GithubAccessTokenResponse;
use super::Github;
use crate::auth::application::port::out::forge::{
    errors::*, refresh_access_token::RefreshAccessToken,
};
use crate::auth::domain::OAuthAccessToken;

#[derive(Clone, Debug, Eq, PartialEq, PartialOrd, Ord, Serialize, Deserialize)]
struct GithubAccessTokenRefreshRequest {
    refresh_token: String,
    client_id: String,
    client_secret: String,
    grant_type: String,
}

impl GithubAccessTokenRefreshRequest {
    pub fn new(refresh_token: String, github: &Github) -> Self {
        Self {
            client_id: github.client_id.clone(),
            client_secret: github.client_secret.clone(),
            grant_type: "refresh_token".into(),
            refresh_token,
        }
    }
}

#[async_trait::async_trait]
impl RefreshAccessToken for Github {
    async fn refresh_access_token(
        &self,
        old_access_token: &OAuthAccessToken,
    ) -> OutForgePortResult<OAuthAccessToken> {
        let payload =
            GithubAccessTokenRefreshRequest::new(old_access_token.refresh_token.clone(), self);

        let u = {
            let mut u = self.url().to_owned();
            u.set_path("/login/oauth/access_token");
            u.set_query(Some(&format!(
                "refresh_token={}&client_id={}&client_secret={}&grant_type={}",
                payload.refresh_token, payload.client_id, payload.client_secret, payload.grant_type,
            )));

            u
        };

        let res: GithubAccessTokenResponse = self
            .http_client
            .post(u)
            .header(reqwest::header::ACCEPT, "application/json")
            .send()
            .await
            .unwrap()
            .json()
            .await
            .unwrap();
        Ok(res.into())
    }
}

#[cfg(test)]
mod tests {
    use url::Url;

    use super::*;

    use crate::auth::application::port::input::forge::refresh_access_token::tests::*;

    #[actix_rt::test]
    async fn test_github_refresh_access_token() {
        let client_id = "statetestpostgres";
        let client_secret = "oauthprovitestpostgres";
        let refresh_token = "refresh_token";

        let mut settings = crate::settings::tests::get_settings().await;

        let mut srv = mockito::Server::new_async().await;
        settings.forges.github.url = Url::parse(&srv.url()).unwrap();

        let old_access_token = OAuthAccessToken {
            access_token: "foo".into(),
            refresh_token: refresh_token.into(),
            expires_in: 9999,
            ..Default::default()
        };

        let new_resp_data = GithubAccessTokenResponse {
            access_token: "new_access_token".into(),
            refresh_token: "new_refresh_token".into(),
            expires_in: 9999,
            ..Default::default()
        };

        let f = Github::new(
            settings.forges.github.url.clone(),
            settings.forges.github.api_url.clone(),
            client_id.into(),
            client_secret.into(),
            mock_refresh_access_token_usecase_factory(None),
        );

        let mock = {
            let payload =
                GithubAccessTokenRefreshRequest::new(old_access_token.refresh_token.clone(), &f);

            srv.mock("POST", format!("/login/oauth/access_token?refresh_token={}&client_id={}&client_secret={}&grant_type={}",
                &payload.refresh_token, payload.client_id, payload.client_secret, payload.grant_type
            ).as_str())
                .with_status(200)
                .with_header("Content-Type", "application/json")
                .with_body(serde_json::to_string(&new_resp_data).unwrap())
                .create_async()
                .await
        };

        let new_access_token = f.refresh_access_token(&old_access_token).await.unwrap();
        let expected_access_token: OAuthAccessToken = new_resp_data.into();
        assert_ne!(old_access_token, new_access_token);
        assert_eq!(expected_access_token, new_access_token);
        mock.assert_async().await;
    }
}
