// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use serde::{Deserialize, Serialize};
use time::OffsetDateTime;
use url::Url;

use super::Github;
use crate::auth::application::port::out::forge::{
    errors::OutForgePortResult, request_access_token::RequestAccessToken,
};
use crate::auth::domain::OAuthAccessToken;

#[derive(Clone, Debug, Default, Eq, PartialEq, PartialOrd, Ord, Serialize, Deserialize)]
pub struct GithubAccessTokenResponse {
    pub refresh_token: String,
    pub access_token: String,
    pub token_type: String,
    pub expires_in: u64,
}

impl From<GithubAccessTokenResponse> for OAuthAccessToken {
    fn from(v: GithubAccessTokenResponse) -> Self {
        let created_at = OffsetDateTime::now_utc();
        Self::new(
            v.refresh_token,
            v.access_token,
            v.token_type,
            v.expires_in,
            created_at,
        )
    }
}

#[derive(Clone, Debug, Eq, PartialEq, PartialOrd, Ord, Serialize, Deserialize)]
struct GithubAccessTokenRequest {
    code: String,
    client_id: String,
    client_secret: String,
    redirect_uri: Url,
}

impl GithubAccessTokenRequest {
    pub fn new(code: String, redirect_uri: Url, github: &Github) -> Self {
        Self {
            code,
            client_id: github.client_id.clone(),
            client_secret: github.client_secret.clone(),
            redirect_uri,
        }
    }
}

#[async_trait::async_trait]
impl RequestAccessToken for Github {
    async fn request_access_token(
        &self,
        code: String,
        redirect_uri: Url,
    ) -> OutForgePortResult<OAuthAccessToken> {
        let payload = GithubAccessTokenRequest::new(code, redirect_uri, self);
        let u = {
            let mut u = self.url().to_owned();
            u.set_path("/login/oauth/access_token");
            u.set_query(Some(&format!(
                "code={}&client_id={}&client_secret={}&redirect_uri={}",
                payload.code, payload.client_id, payload.client_secret, payload.redirect_uri,
            )));
            u
        };

        let res = self
            .http_client
            .post(u)
            .header(reqwest::header::ACCEPT, "application/json")
            .send()
            .await
            .unwrap();
        let res: GithubAccessTokenResponse = res.json().await.unwrap();

        Ok(res.into())
    }
}

#[cfg(test)]
mod tests {
    use super::*;

    use crate::auth::application::port::input::forge::refresh_access_token::tests::*;

    #[actix_rt::test]
    async fn test_github_request_access_token() {
        let client_id = "statetestpostgres";
        let client_secret = "oauthprovitestpostgres";
        let code = "oauthprovitestpostgres";
        let redirect_uri = Url::parse("https://oauthprovitestpostgres").unwrap();

        let mut settings = crate::settings::tests::get_settings().await;

        let mut srv = mockito::Server::new_async().await;
        settings.forges.github.url = Url::parse(&srv.url()).unwrap();

        let data = GithubAccessTokenResponse {
            access_token: "foo".into(),
            expires_in: 9999,
            ..Default::default()
        };

        let f = Github::new(
            settings.forges.github.url.clone(),
            settings.forges.github.api_url.clone(),
            client_id.into(),
            client_secret.into(),
            mock_refresh_access_token_usecase_factory(None),
        );

        let mock = {
            let payload = GithubAccessTokenRequest::new(code.into(), redirect_uri.clone(), &f);

            srv.mock("POST", format!("/login/oauth/access_token?code={}&client_id={}&client_secret={}&redirect_uri={}",
                    payload.code,payload.client_id, payload.client_secret, payload.redirect_uri).as_str())
                .with_status(200)
                .with_header("Content-Type", "application/json")
                .with_body(serde_json::to_string(&data).unwrap())
                .create_async()
                .await
        };

        let access_token = f
            .request_access_token(code.into(), redirect_uri)
            .await
            .unwrap();
        let expected_access_token: OAuthAccessToken = data.into();
        assert_eq!(expected_access_token, access_token);
        mock.assert_async().await;
    }
}
