// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use actix_web::{
    http::{header, StatusCode},
    HttpResponse, HttpResponseBuilder, ResponseError,
};
use derive_more::Display;
use serde::{Deserialize, Serialize};

use crate::auth::application::services::{
    process_authorization_response::errors::ProcessAuthorizationServiceError,
    request_authorization::errors::RequestAuthorizationServiceError,
};

pub type InUIResult<V> = Result<V, InUIError>;

#[derive(Debug, Display, Clone, Serialize, Deserialize, PartialEq, Eq, PartialOrd, Ord)]
pub enum InUIError {
    InternalServerError,
    BadRequest,
}

impl From<ProcessAuthorizationServiceError> for InUIError {
    fn from(v: ProcessAuthorizationServiceError) -> Self {
        match v {
            ProcessAuthorizationServiceError::InternalError => Self::InternalServerError,
            ProcessAuthorizationServiceError::BadRequest => Self::BadRequest,
        }
    }
}

impl From<RequestAuthorizationServiceError> for InUIError {
    fn from(v: RequestAuthorizationServiceError) -> Self {
        match v {
            RequestAuthorizationServiceError::InternalError => Self::InternalServerError,
        }
    }
}

impl ResponseError for InUIError {
    fn error_response(&self) -> HttpResponse {
        HttpResponseBuilder::new(self.status_code())
            .append_header((header::CONTENT_TYPE, "application/sjon; charset=UTF-8"))
            .body(serde_json::to_string(self).unwrap())
    }

    fn status_code(&self) -> actix_web::http::StatusCode {
        match self {
            Self::BadRequest => StatusCode::BAD_REQUEST,
            Self::InternalServerError => StatusCode::INTERNAL_SERVER_ERROR,
        }
    }
}
