// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use std::sync::Arc;

use crate::auth::application::port::out::db::{
    delete_oauth_access_token::DeleteOAuthAccessToken, get_oauth_access_token::GetOAuthAccessToken,
    save_oauth_access_token::SaveOAuthAccessToken,
};
use crate::auth::application::port::out::forge::refresh_access_token::RefreshAccessToken;

use super::{errors::*, RefreshAccessTokenUseCase};

#[derive(Clone)]
pub struct RefreshAccessTokenService {
    save_oauth_access_token_adapter: Arc<dyn SaveOAuthAccessToken>,
    get_oauth_access_token_adapter: Arc<dyn GetOAuthAccessToken>,
    delete_oauth_access_token_adapter: Arc<dyn DeleteOAuthAccessToken>,
    refresh_access_token_adapter: Arc<dyn RefreshAccessToken>,
}

impl RefreshAccessTokenService {
    pub fn new(
        save_oauth_access_token_adapter: Arc<dyn SaveOAuthAccessToken>,
        get_oauth_access_token_adapter: Arc<dyn GetOAuthAccessToken>,
        delete_oauth_access_token_adapter: Arc<dyn DeleteOAuthAccessToken>,
        refresh_access_token_adapter: Arc<dyn RefreshAccessToken>,
    ) -> Self {
        Self {
            save_oauth_access_token_adapter,
            get_oauth_access_token_adapter,
            delete_oauth_access_token_adapter,
            refresh_access_token_adapter,
        }
    }
}

#[async_trait::async_trait]
impl RefreshAccessTokenUseCase for RefreshAccessTokenService {
    async fn refresh_access_token(
        &self,
        cmd: super::command::RefreshAccessTokenCommand,
    ) -> RefreshAccessTokenServiceResult<()> {
        let old_access_token = self
            .get_oauth_access_token_adapter
            .get_oauth_access_token(cmd.username(), cmd.oauth_provider())
            .await?;
        let new_access_token = self
            .refresh_access_token_adapter
            .refresh_access_token(&old_access_token)
            .await?;

        self.delete_oauth_access_token_adapter
            .delete_oauth_access_token(cmd.username(), cmd.oauth_provider())
            .await?;
        self.save_oauth_access_token_adapter
            .save_oauth_access_token(cmd.username(), cmd.oauth_provider(), &new_access_token)
            .await?;

        Ok(())
    }
}

#[cfg(test)]
mod tests {
    use crate::auth::application::{
        port::out::{
            db::{
                delete_oauth_access_token::tests::*, get_oauth_access_token::tests::*,
                save_oauth_access_token::tests::*,
            },
            forge::refresh_access_token::tests::*,
        },
        services::refresh_access_token::command::RefreshAccessTokenCommand,
    };
    use crate::tests::bdd::*;

    use super::*;

    #[actix_rt::test]
    async fn test_service() {
        let oauth_provider = "test_service_request_auth_oauth_provider";
        let username = "refreshaccessusernameservice";

        let mock_save_oauth_access_token = mock_save_oauth_access_token(IS_CALLED_ONLY_ONCE);
        let mock_get_oauth_access_token = mock_get_oauth_access_token(IS_CALLED_ONLY_ONCE);
        let mock_delete_oauth_access_token = mock_delete_oauth_access_token(IS_CALLED_ONLY_ONCE);
        let mock_refresh_access_token = mock_refresh_access_token(IS_CALLED_ONLY_ONCE);

        let s = RefreshAccessTokenService::new(
            mock_save_oauth_access_token,
            mock_get_oauth_access_token,
            mock_delete_oauth_access_token,
            mock_refresh_access_token,
        );

        let cmd =
            RefreshAccessTokenCommand::new_command(username.into(), oauth_provider.into()).unwrap();

        s.refresh_access_token(cmd).await.unwrap();
    }
}
