// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use config::{builder::DefaultState, ConfigBuilder};
use serde::Deserialize;
use std::env;

#[derive(Debug, Clone, Deserialize, Eq, PartialEq)]
pub struct Server {
    pub port: u32,
    pub domain: String,
    pub ip: String,
    pub cookie_secret: String,
}

impl Server {
    pub fn get_ip(&self) -> String {
        format!("{}:{}", self.ip, self.port)
    }

    pub fn env_override(mut s: ConfigBuilder<DefaultState>) -> ConfigBuilder<DefaultState> {
        for (parameter, env_var_name) in [
            ("server.port", "PORT"),
            ("server.domain", "FORGEFLUX_server_DOMAIN"),
            ("server.cookie_secret", "FORGEFLUX_server_COOKIE_SECRET"),
            ("server.ip", "FORGEFLUX_server_IP"),
        ]
        .iter()
        {
            if let Ok(val) = env::var(env_var_name) {
                log::debug!("Overriding [{parameter}] with environment variable {env_var_name}");
                s = s.set_override(parameter, val).unwrap();
            }
        }

        s
    }
}

#[cfg(test)]
mod tests {
    use crate::env_helper;

    use super::*;

    #[test]
    fn test_server_env_override() {
        let init_settings = crate::settings::Settings::new().unwrap();
        env_helper!(init_settings, "PORT", 22, server.port);
        env_helper!(
            init_settings,
            "FORGEFLUX_server_DOMAIN",
            "test_server_env_override.org",
            server.domain
        );
        env_helper!(init_settings, "FORGEFLUX_server_IP", "1.1.1.1", server.ip);
        env_helper!(
            init_settings,
            "FORGEFLUX_server_COOKIE_SECRET",
            "asdfasdflkjhasdlkfhalksdf",
            server.cookie_secret
        );
    }
}
